@@ -24,13 +24,16 @@ namespace defaults {
   constexpr const bool disableSA{false};
   constexpr const float consensusSlack{0.0};
   constexpr const double minScoreFraction{0.65};
+  constexpr const double pre_merge_chain_sub_thresh{0.75};
+  constexpr const double post_merge_chain_sub_thresh{0.9};
+  constexpr const double orphan_chain_sub_thresh{0.95};
   constexpr const double scoreExp{1.0};
   constexpr const int16_t matchScore{2};
   constexpr const int16_t mismatchPenalty{-4};
-  constexpr const int16_t gapOpenPenalty{4};
+  constexpr const int16_t gapOpenPenalty{6};
   constexpr const int16_t gapExtendPenalty{2};
   constexpr const int32_t dpBandwidth{15};
-  constexpr const uint32_t mismatchSeedSkip{5};
+  constexpr const uint32_t mismatchSeedSkip{3};
   constexpr const bool disableChainingHeuristic{false};
   constexpr const bool eqClassMode{false};
   constexpr const bool hardFilter{false};
@@ -143,11 +146,13 @@ namespace defaults {
   constexpr const bool dumpFeatures{false};
   constexpr const bool dumpBFH{false};
   constexpr const bool dumpUmiGraph{false};
+  constexpr const bool dumpCellEq{false};
   constexpr const bool dumpMtx{false};
   constexpr const bool noEM{false};
   constexpr const bool debug{true};
   constexpr const uint32_t trimRight{0};
   constexpr const uint32_t numBootstraps{0};
+  constexpr const uint32_t numGibbsSamples{0};
   constexpr const uint32_t lowRegionMinNumBarcodes{200};
   constexpr const uint32_t maxNumBarcodes{100000};
   constexpr const uint32_t expectCells{0};
