@@ -20,7 +20,6 @@ extern "C" {
 #include <memory>
 #include <mutex>
 #include <random>
-#include <tbb/atomic.h>
 #include <thread>
 #include <unordered_map>
 #include <unordered_set>
@@ -99,7 +98,6 @@ void sampleMiniBatch(AlignmentLibraryT<FragT>& alnLib,
   using salmon::math::logAdd;
   using salmon::math::logSub;
 
-  bool useFSPD{salmonOpts.useFSPD};
   bool noLengthCorrection{salmonOpts.noLengthCorrection};
   bool useFragLengthDist{!salmonOpts.noFragLengthDist};
   bool useAuxParams = (processedReads >= salmonOpts.numPreBurninFrags);
@@ -258,11 +256,6 @@ void sampleMiniBatch(AlignmentLibraryT<FragT>& alnLib,
             // Allow for a non-uniform fragment start position distribution
             double startPosProb = -logRefLength;
             auto hitPos = aln->left();
-            if (useFSPD and burnedIn and hitPos < refLength) {
-              auto& fragStartDist =
-                  fragStartDists[transcript.lengthClassIndex()];
-              startPosProb = fragStartDist(hitPos, refLength, logRefLength);
-            }
 
             double auxProb = startPosProb + logFragProb + aln->logQualProb() +
                              errLike + logAlignCompatProb;
