@@ -563,6 +563,7 @@ namespace alevin {
       auto logPath = aopt.outputDirectory / "alevin.log";
       auto fileSink = std::make_shared<spdlog::sinks::simple_file_sink_mt>(logPath.string(), true);
       auto consoleSink = std::make_shared<spdlog::sinks::ansicolor_stderr_sink_mt>();
+      consoleSink->set_color(spdlog::level::warn, consoleSink->magenta);
       std::vector<spdlog::sink_ptr> sinks{consoleSink, fileSink};
       aopt.jointLog = spdlog::create("alevinLog", std::begin(sinks), std::end(sinks));
 
@@ -578,8 +579,10 @@ namespace alevin {
       aopt.dumpBarcodeEq = vm["dumpBarcodeEq"].as<bool>();
       aopt.dumpBFH = vm["dumpBfh"].as<bool>();
       aopt.dumpUmiGraph = vm["dumpUmiGraph"].as<bool>();
+      aopt.dumpCellEq = vm["dumpCellEq"].as<bool>();
       aopt.trimRight = vm["trimRight"].as<uint32_t>();
       aopt.numBootstraps = vm["numCellBootstraps"].as<uint32_t>();
+      aopt.numGibbsSamples = vm["numCellGibbsSamples"].as<uint32_t>();
       aopt.lowRegionMinNumBarcodes = vm["lowRegionMinNumBarcodes"].as<uint32_t>();
       aopt.maxNumBarcodes = vm["maxNumBarcodes"].as<uint32_t>();
       aopt.freqThreshold = vm["freqThreshold"].as<uint32_t>();
@@ -602,6 +605,12 @@ namespace alevin {
         return false;
       }
 
+      if (aopt.numGibbsSamples > 0 and aopt.numBootstraps > 0) {
+        aopt.jointLog->error("Either of --numCellGibbsSamples or --numCellBootstraps "
+                             "can be used");
+        return false;
+      }
+
       if ( aopt.numBootstraps > 0 and aopt.noEM ) {
         aopt.jointLog->error("cannot perform bootstrapping with noEM option.");
         return false;
