@@ -30,6 +30,9 @@ GZipWriter::~GZipWriter() {
   if (cellEQStream_){
     cellEQStream_->reset();
   }
+  if (cellDedupEQStream_){
+    cellDedupEQStream_->reset();
+  }
   if (umiGraphStream_){
     umiGraphStream_->reset();
   }
@@ -69,6 +72,9 @@ void GZipWriter::close_all_streams(){
   if (cellEQStream_){
     cellEQStream_->reset();
   }
+  if (cellDedupEQStream_){
+    cellDedupEQStream_->reset();
+  }
   if (umiGraphStream_){
     umiGraphStream_->reset();
   }
@@ -401,6 +407,8 @@ bool GZipWriter::writeEmptyMeta(const SalmonOpts& opts, const ExpT& experiment,
     oa(cereal::make_nvp("library_types", libStrings));
 
     oa(cereal::make_nvp("frag_dist_length", 0));
+    oa(cereal::make_nvp("frag_length_mean", 0.0));
+    oa(cereal::make_nvp("frag_length_sd", 0.0));
     oa(cereal::make_nvp("seq_bias_correct", false));
     oa(cereal::make_nvp("gc_bias_correct", false));
     oa(cereal::make_nvp("num_bias_bins", 0));
@@ -565,9 +573,9 @@ bool GZipWriter::writeMeta(const SalmonOpts& opts, const ExpT& experiment, const
 
   bfs::path fldPath = auxDir / "fld.gz";
   int32_t numFLDSamples{10000};
-  auto fldSamples = distribution_utils::samplesFromLogPMF(
+  auto fldSummary = distribution_utils::samplesFromLogPMF(
       experiment.fragmentLengthDistribution(), numFLDSamples);
-  writeVectorToFile(fldPath, fldSamples);
+  writeVectorToFile(fldPath, fldSummary.samples);
 
   bfs::path normBiasPath = auxDir / "expected_bias.gz";
   writeVectorToFile(normBiasPath, experiment.expectedSeqBias());
@@ -768,7 +776,9 @@ bool GZipWriter::writeMeta(const SalmonOpts& opts, const ExpT& experiment, const
     oa(cereal::make_nvp("num_libraries", libStrings.size()));
     oa(cereal::make_nvp("library_types", libStrings));
 
-    oa(cereal::make_nvp("frag_dist_length", fldSamples.size()));
+    oa(cereal::make_nvp("frag_dist_length", fldSummary.samples.size()));
+    oa(cereal::make_nvp("frag_length_mean", fldSummary.mean));
+    oa(cereal::make_nvp("frag_length_sd", fldSummary.sd));
     oa(cereal::make_nvp("seq_bias_correct", opts.biasCorrect));
     oa(cereal::make_nvp("gc_bias_correct", opts.gcBiasCorrect));
     oa(cereal::make_nvp("num_bias_bins", bcounts.size()));
@@ -1481,6 +1491,49 @@ bool GZipWriter::writeCellEQVec(size_t barcode, const std::vector<uint32_t>& off
   return true;
 }
 
+// TODO: complete this method
+bool GZipWriter::writeDedupCellEQVec(
+  size_t barcode, 
+  const std::vector<std::vector<uint32_t>>& labels,
+  const std::vector<uint32_t>& counts,
+  bool quiet
+  ) {
+#if defined __APPLE__
+  spin_lock::scoped_lock sl(writeMutex_);
+#else
+  std::lock_guard<std::mutex> lock(writeMutex_);
+#endif
+  if (!cellDedupEQStream_) {
+    cellDedupEQStream_.reset(new boost::iostreams::filtering_ostream);
+    cellDedupEQStream_->push(boost::iostreams::gzip_compressor(6));
+    auto ceqFilename = path_ / "alevin" / "cell_dedup_eq_mat.gz";
+    cellDedupEQStream_->push(boost::iostreams::file_sink(ceqFilename.string(),
+                                                    std::ios_base::out | std::ios_base::binary));
+  }
+  
+  boost::iostreams::filtering_ostream& ofile = *cellDedupEQStream_;
+  size_t num = labels.size();
+  size_t elSize = sizeof(typename std::vector<uint32_t>::value_type);
+  // write the barcode
+  ofile.write(reinterpret_cast<char*>(&barcode), sizeof(barcode));
+  // write the number of elements in the list
+  ofile.write(reinterpret_cast<char*>(&num), sizeof(barcode));
+  // write the individual labels
+  for (size_t i = 0 ; i < num ; ++i) {
+    size_t labelLength = labels[i].size();
+    // write the length of the equivalence class vector
+    ofile.write(reinterpret_cast<char*>(&labelLength), sizeof(barcode));
+    // write the label vector
+    ofile.write(reinterpret_cast<char*>(const_cast<uint32_t*>(labels[i].data())), elSize * labelLength);
+  }
+  ofile.write(reinterpret_cast<char*>(const_cast<uint32_t*>(counts.data())), elSize * num);
+  // write the offsets and counts
+  if (!quiet) {
+    logger_->info("wrote EQ vector for barcode ID {}", barcode);
+  }
+  return true;
+}
+
 bool GZipWriter::writeUmiGraph(alevin::graph::Graph& g,
                                std::string& cbString) {
 #if defined __APPLE__
